<?php

namespace EventPortal\Tests\Unit\Agenda\Actions;

use EventPortal\Domain\Agenda\Actions\DeleteAgenda;
use EventPortal\Domain\Agenda\Models\Agenda;
use EventPortal\Domain\Agenda\Models\Pathway;
use EventPortal\Domain\Agenda\Models\Session;
use EventPortal\Domain\Event\Models\EventSpeaker;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;

class DeleteAgendaTest extends TestCase
{
    use RefreshDatabase;

    /** @test **/
    public function can_delete_agenda()
    {
        $agenda = factory(Agenda::class)->create();

        $pathwayA = factory(Pathway::class)->create([
            'pathwayable_id' => $agenda->id,
            'pathwayable_type' => Agenda::class,
        ]);
        $sessionA = factory(Session::class)->create(['pathway_id' => $pathwayA->id]);
        $sessionB = factory(Session::class)->create(['pathway_id' => $pathwayA->id]);

        $pathwayB = factory(Pathway::class)->create([
            'pathwayable_id' => $agenda->id,
            'pathwayable_type' => Agenda::class,
        ]);
        $sessionC = factory(Session::class)->create(['pathway_id' => $pathwayB->id]);
        $sessionD = factory(Session::class)->create(['pathway_id' => $pathwayB->id]);

        (new DeleteAgenda)->execute($agenda);

        $this
            ->assertDatabaseMissing('agendas', [
                'id' => $agenda->id,
            ])
            ->assertDatabaseMissing('pathways', [
                'id' => $pathwayA->id,
                'pathwayable_id' => $agenda->id,
                'pathwayable_type' => Agenda::class,
            ])
            ->assertDatabaseMissing('sessions', [
                'id' => $sessionA->id,
                'pathway_id' => $pathwayA->id,
            ])
            ->assertDatabaseMissing('sessions', [
                'id' => $sessionB->id,
                'pathway_id' => $pathwayA->id,
            ])
            ->assertDatabaseMissing('pathways', [
                'id' => $pathwayB->id,
                'pathwayable_id' => $agenda->id,
                'pathwayable_type' => Agenda::class,
            ])
            ->assertDatabaseMissing('sessions', [
                'id' => $sessionC->id,
                'pathway_id' => $pathwayB->id,
            ])
            ->assertDatabaseMissing('sessions', [
                'id' => $sessionD->id,
                'pathway_id' => $pathwayB->id,
            ]);
    }

    /** @test **/
    public function deleting_agenda_removes_speakers()
    {
        $agenda = factory(Agenda::class)->create();

        $pathway = factory(Pathway::class)->create([
            'pathwayable_id' => $agenda->id,
            'pathwayable_type' => Agenda::class,
        ]);
        $session = factory(Session::class)->create(['pathway_id' => $pathway->id]);
        $eventSpeaker = factory(EventSpeaker::class)->create(['event_id' => $agenda->event->id]);

        $session->eventSpeakers()->attach([$eventSpeaker->id]);

        (new DeleteAgenda)->execute($agenda);

        $this
            ->assertDatabaseMissing('agendas', [
                'id' => $agenda->id,
            ])
            ->assertDatabaseMissing('pathways', [
                'id' => $pathway->id,
                'pathwayable_id' => $agenda->id,
                'pathwayable_type' => Agenda::class,
            ])
            ->assertDatabaseMissing('sessions', [
                'id' => $session->id,
                'pathway_id' => $pathway->id,
            ])
            ->assertDatabaseMissing('event_speaker_session', [
                'event_speaker_id' => $eventSpeaker->id,
                'session_id' => $session->id,
            ]);
    }
}
